clear all;
confs= {'confFemale60', ...
        'confMale65', ...
        'confFemale65', ...
        'confMale60', ...
        };

BASEDIR = '..'; % path to base directory (contains data, estimate, etc as
                % subdirs) 
YEAR = 92;
SEPARATE = true; % separate by year
GAM = 3;
BGAM = 3;
SIMULATE = true;
USEOLDSIM = false;
OBJECTIVE = 'mle';
READLAMBDA = false; 
LAMBDA = 0.1102206337978725; % pooled value
FRAC = 0.2;
DIST = 'normal';
DEGREE = 2 ;
ZETA = 1;
ESTIMATE = true;
INCLUDEX = false;
FASTHAZARD = 1;
EXTERNAL = false;
PUBLICANNUITY = 0; % put this much wealth in a (alpha-specific)
                     % actuarially fair annuity
%RATE = 0.0; % riskless interest rate, if don't want to use one in data
%INFL = .049779; % inflation rate 
DUMB.p = 0.0; % this percent of people just pick the middle
DUMB.dmean = false; % dumb people have different mean alpha
DUMB.dsig = false; % dumbe people have different sigma alpha
DUMB.estP = false;

RAHETERO = false;

NINT = 12;

EXTRALAGRID = (-6.505:0.01:-4.505)';

path(path,'cutoffs');

for c = 1:4
  clear est;
  clear annuityLike;
  clear step2Like;
  clear mortLike;
  clear transformParam;
  configName = confs{c};
  eval(['cfg = ' configName '();']);
  mortOnly = strcmp(OBJECTIVE,'mlmort'); 
  if (mortOnly)  % change config to just do mortality
    READLAMBDA = false;
  end
  cfg.est.objective = OBJECTIVE;
  PENALTY = 100;

  cfg.est.disp = 50;

  switch(OBJECTIVE)
   case '2step'
    cfg.outprefix = [cfg.outprefix '_2step'];
   case 'mlmort'
    cfg.outprefix = [cfg.outprefix 'Mort'];
  end

  cfg.cut.gam = GAM;
  cfg.cut.bgam = BGAM;
  cfg.cut.fracAnnuitized = FRAC;
  cfg.est.dist = DIST;
  cfg.cut.zeta = ZETA;
  cfg.mort.fasthazard = FASTHAZARD;
  cfg.cut.fasthazard = FASTHAZARD;
  cfg.cut.public = PUBLICANNUITY;
  cfg.est.dumb = DUMB;
  
  
  if (SEPARATE) 
    cfg.outprefix = [cfg.outprefix '_just' num2str(YEAR)];
  else
    if (YEAR~=92)
      cfg.outprefix = [cfg.outprefix '_' num2str(YEAR)];
    end
  end
  if (GAM~=3)
    cfg.outprefix = [cfg.outprefix 'g' num2str(GAM)];
  end
  if (BGAM~=GAM)
    cfg.outprefix = [cfg.outprefix '_bg' num2str(BGAM)];
  end
  if (FRAC~=0.2)
    cfg.outprefix = [cfg.outprefix '_frac' num2str(100*FRAC)];
  end
  if (~strcmp(DIST,'normal'))
    cfg.outprefix = [cfg.outprefix,'_' DIST];
  end
  if(ZETA~=1)
    cfg.outprefix = [cfg.outprefix,'_z' num2str(1/ZETA-1)];
  end
  if(INCLUDEX)
    cfg.outprefix = [cfg.outprefix, '_X'];
    cfg.data.lalphaX = [-1 5 7];
    cfg.data.alphaXName(2).s = 'Premium';
    cfg.data.alphaXName(3).s = 'Percqua';

    cfg.data.lbetaX = [-1 5 7];
    cfg.data.betaXName(2).s = 'Premium';
    cfg.data.betaXName(3).s = 'Percqua';
  end
  if(FASTHAZARD~=1)
    cfg.outprefix = [cfg.outprefix, '_t' num2str(FASTHAZARD)];
  end
  if(EXTERNAL)
    cfg.outprefix = [cfg.outprefix '_ext'];
    cfg.data.okay = 15;
    cfg.data.okeep = 1;
  end
  if(PUBLICANNUITY)
    cfg.outprefix = [cfg.outprefix '_pub' num2str(PUBLICANNUITY*100)];
  end
  if(exist('RATE','var') && RATE>0)
    cfg.outprefix = [cfg.outprefix '_r' num2str(RATE*100)];
    cfg.cut.rho = 1/(1+RATE);
    cfg.cut.delta = 1/(1+RATE);
    cfg.data = rmfield(cfg.data,'lrate');
  end
  if(exist('INFL','var') && INFL>0)
    cfg.outprefix = [cfg.outprefix '_inf' num2str(INFL*100)];
    cfg.cut.inflation = INFL;
    cfg.data = rmfield(cfg.data,'linfl');
  end
  if (DUMB.p>0 || DUMB.estP)
    if (DUMB.estP)
      cfg.outprefix = [cfg.outprefix '_dumbEst'];
    else
      cfg.outprefix = [cfg.outprefix '_dumb' num2str(DUMB.p*100)];
    end
  end
  
  if (RAHETERO)
    file = regexprep(cfg.outprefix,'\.(\d)','p$1');
    scfg = cfg;
    load([file 'Results']);
    cfg = scfg;
    clear scfg;
    cfg.cut.raHetero.beta = exp(est.parm.gBeta);
    cfg.outprefix = [cfg.outprefix '_raHetero'];
  end
  
  cfg.outprefix = regexprep(cfg.outprefix,'\.(\d)','p$1');

  % read data
  if (SEPARATE) 
    switch YEAR
     case {92,95}
      cfg.data.name = [BASEDIR '/data/newCompB2_9295.txt'];
     case {90,97} 
      cfg.data.name = [BASEDIR '/data/newCompB2_9097.txt']; 
     otherwise
      error(['no appropriate data set for separate and ' num2str(YEAR)]);
    end
      cfg.data.keepYear = 1900 + YEAR;
  else
    if (YEAR<100)
      cfg.data.name = [BASEDIR '/data/newCompB2_19' num2str(YEAR) ...
                       '.txt'];
    else
      cfg.data.name = [BASEDIR '/data/newCompB2_' num2str(YEAR) ...
                       '.txt'];
    end
    cfg.data.allDummies=false;
  end
  
  if (READLAMBDA)
    if(SEPARATE)
      if (YEAR<=94)
        ystr = '92';
      else
        ystr = '95';
      end
      filename = [confs{c} '_' ystr];
    else
      filename = confs{c};
    end
    if(FASTHAZARD~=1)
      filename = [filename '_t' num2str(FASTHAZARD)];
    end
    load ([filename '.lambda'])
    eval(['cfg.mort.lambda = ' filename]);
  end
  if (exist('LAMBDA','var'))
    cfg.outprefix = [cfg.outprefix '_lambda' num2str(LAMBDA)];    
    cfg.mort.lambda=LAMBDA;
    cfg.cut.lambda = cfg.mort.lambda;
  end

  [data cfg] = readData(cfg); % see readData for details of data struc
  if (~strcmp(cfg.est.objective,'mlmort'))
    %cfg.cut.redo = true;
    % load cutoffs
    if (EXTRALAGRID)
      cfg.cut.la = sort([cfg.cut.la; EXTRALAGRID]);
    end
    cfg.cut.lambda = cfg.mort.lambda;
    cut = loadcutoffs(cfg);
    cut = cleanCut(cut,cfg,data);
    est.cut = cut;
  end
  if (isfield(cfg,'dropBadGroups') && cfg.dropBadGroups)
    [data cut cfg] = dropBadGroups(data,cut,cfg);
    est.cut = cut;
  end  
  
  writeAMPLdata(data,cfg,cut);
  Data{c} = data;
  Cut{c} = cut;
  Cfg{c} = cfg;
  continue;
end;

% write data set with all four pooled
Cfg{1}.outprefix = '/pooled';
writeAMPLdata(Data,Cfg,Cut);
